use axum::{
    extract::ws::{WebSocket, WebSocketUpgrade},
    response::Html,
    routing::get,
    Router,
};
use tokio::{
    net::TcpListener,
    time::{self, Duration},
};

#[tokio::main]
async fn main() -> std::io::Result<()> {
    let app = Router::new()
        .route("/", get(get_index))
        .route("/ws", get(get_ws));

    let listener = TcpListener::bind("localhost:8080").await?;
    axum::serve(listener, app).await
}

async fn get_index() -> Html<&'static str> {
    Html(
        r#"<!doctype html>
        <html>
            <body>
                <p>
                    Welcome to boulder, just southwest of Longmont!
                    <br />
                    (someone who is good at web design please fix this)
                </p>
                <p>Messages from the server:</p>
                <ul id="message-history"></ul>
            </body>
        </html>
        <script type="text/javascript">
            const logs = document.getElementById("message-history");

            const wsUrl = new URL(document.location);
            wsUrl.protocol = "ws:";
            wsUrl.pathname = "/ws";
            console.log("opening websocket connection", wsUrl);
            const ws = new WebSocket(wsUrl);

            ws.addEventListener("message", (event) => {
                console.log("we just got a message!", event);
                if (typeof event.data === "string") {
                    const log = document.createElement("li");
                    log.textContent = event.data;
                    logs.appendChild(log);
                }
            });
        </script>"#,
    )
}

async fn get_ws(ws: WebSocketUpgrade) -> axum::Response {
    ws.on_upgrade(example_websocket)
}

async fn example_websocket(mut ws: WebSocket) {
    let mut timer = time::interval(Duration::from_millis(500));
    for sound in ["nya", "mrrp", "waf", "yip?"].into_iter().cycle() {
        timer.tick().await;
        if let Err(error) = ws.send(sound.into()).await {
            eprintln!("client disconnected: {error:?}");
            break;
        }
    }
}
